#!perl

use Test::More tests => 8;

use_ok("Devel::Cover::Report::Clover");
use_ok("Devel::Cover::Report::Clover::Builder");
use_ok("Devel::Cover::Report::Clover::Class");
use_ok("Devel::Cover::Report::Clover::File");
use_ok("Devel::Cover::Report::Clover::FileRegistry");
use_ok("Devel::Cover::Report::Clover::Package");
use_ok("Devel::Cover::Report::Clover::Project");
use_ok("Devel::Cover::Report::Clover::Reportable");
